# Copyright 2021 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""Module for configuring machine learning models"""
from dataclasses import dataclass


@dataclass
class Configuration:
    """
    A model configuration.
    Typically sub-classes of a configuration parser implement
    this class to provide a mechanism to feed parameters into models or training.
    """

    def __init__(self) -> None:
        """
        Constructor creates a new instance.
        Ths constructor must be called by derived sub-classes.
        """
        pass


@dataclass
class BaseDetectorConfig:
    MODEL_TYPE: str = ""
    MODEL_VERSION: str = ""
    MODEL_SPECIFIER: str = ""


@dataclass
class BaseDetectorInferenceConfig:
    config_path: str
    checkpoint_path: str
    score_threshold: float = 0.5



# Copyright 2021 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""
Module for parsing information from yaml in python accessible attributes for the
AnnotationHandler class.
"""
import logging
import os
from typing import List, Optional

import related
from config_builder import BaseConfigClass

from mlcvzoo_base.configuration.class_mapping_config import ClassMappingConfig
from mlcvzoo_base.configuration.reduction_mapping_config import ReductionMappingConfig
from mlcvzoo_base.data_preparation.structs import (
    CSVOutputStringFormats,
    MOTChallengeFormats,
)

logger = logging.getLogger(__name__)


@related.mutable(strict=True)
class AnnotationHandlerPASCALVOCInputDataConfig(BaseConfigClass):
    """Class for parsing information of PASCAL VOC input format"""

    input_image_dir: str = related.StringField()
    input_xml_dir: str = related.StringField()
    input_sub_dirs: List[str] = related.SequenceField(str)

    image_format: str = related.StringField()
    use_difficult: bool = related.BooleanField(default=True)
    ignore_missing_images: bool = related.BooleanField(default=False)


@related.mutable(strict=True)
class AnnotationHandlerMOTInputDataConfig(BaseConfigClass):
    """Class for parsing MOT information of input data"""

    # Path to the annotation file
    annotation_path: str = related.StringField()
    # Root directory which fits to the relative directory in the given annotation file
    image_dir: str = related.StringField()
    # considered image file format
    image_format: str = related.StringField(required=False, default=".jpg")
    # specifies which format of which challenge/year should be used
    mot_format: str = related.StringField(required=False, default=MOTChallengeFormats.MOT20.value)
    # specifies how to interpret confidence scores in annotations
    ground_truth: bool = related.BooleanField(required=False, default=True)

    def check_values(self) -> bool:
        return self.mot_format in [f.value for f in MOTChallengeFormats]


@related.mutable(strict=True)
class AnnotationHandlerSingleFileInputDataConfig(BaseConfigClass):
    """Class for parsing information of input data"""

    # Path to the annotation file
    input_path: str = related.StringField()
    # Root directory which fits to the relative directory in the given annotation file
    input_root_dir: str = related.StringField()

    # TODO: don't set defaults here?
    use_difficult: bool = related.BooleanField(default=True)


@related.mutable(strict=True)
class AnnotationHandlerWriteOutputCSVAnnotationConfig(BaseConfigClass):
    """Class for parsing information of output format when writing CSV annotations"""

    csv_dir: str = related.StringField()
    csv_base_file_name: str = related.StringField(default="")

    csv_split_0_file_name: str = related.StringField(default="")
    csv_split_1_file_name: str = related.StringField(default="")

    # TODO: add hint for file-name replacement? Add specific config class?
    timestamp_format: str = related.StringField(default="%Y-%m-%d")
    include_surrounding_bboxes: bool = related.BooleanField(default=True)
    output_string_format: str = related.StringField(default=CSVOutputStringFormats.BASE)

    def check_values(self) -> bool:
        if not os.path.isdir(self.csv_dir):
            # Note: this is not necessarily required or
            #       already existing when config is read (see test)
            logger.info("Class='%s', specify correct output_dir'%s'", type(self), self.csv_dir)

        return self.output_string_format in CSVOutputStringFormats.get_values_as_list(
            class_type=CSVOutputStringFormats
        )


@related.mutable(strict=True)
class AnnotationHandlerWriteOutputDarknetAnnotationConfig(BaseConfigClass):
    """Class for parsing information of output format when writing Darknet annotations"""

    train_data_set_dir: str = related.StringField()
    base_file_name: str = related.StringField(default="")
    write_as_symlink: bool = related.BooleanField(default=False)

    def get_train_file_path(self) -> str:
        return os.path.join(self.train_data_set_dir, f"{self.base_file_name}_train.txt")

    def get_test_file_path(self) -> str:
        return os.path.join(self.train_data_set_dir, f"{self.base_file_name}_test.txt")


@related.mutable(strict=True)
class AnnotationHandlerWriteOutputBoxSnippetsConfig(BaseConfigClass):
    """Class for parsing information of output directory when writing annotations"""

    output_dir: str = related.StringField()


@related.mutable(strict=True)
class AnnotationHandlerWriteOutputConfig(BaseConfigClass):
    """Class for parsing information of output format when writing annotations"""

    use_cross_val: bool = related.BooleanField()
    number_splits: int = related.IntegerField()
    split_size: int = related.FloatField()

    csv_annotation: Optional[AnnotationHandlerWriteOutputCSVAnnotationConfig] = related.ChildField(
        cls=AnnotationHandlerWriteOutputCSVAnnotationConfig,
        required=False,
        default=None,
    )

    darknet_train_set: Optional[
        AnnotationHandlerWriteOutputDarknetAnnotationConfig
    ] = related.ChildField(
        cls=AnnotationHandlerWriteOutputDarknetAnnotationConfig,
        required=False,
        default=None,
    )
    darknet_annotation: AnnotationHandlerWriteOutputDarknetAnnotationConfig = related.ChildField(
        cls=AnnotationHandlerWriteOutputDarknetAnnotationConfig,
        required=False,
        default=None,
    )
    use_difficult: bool = related.BooleanField(default=True)
    use_occluded: bool = related.BooleanField(default=True)

    bbox_snippets: Optional[AnnotationHandlerWriteOutputBoxSnippetsConfig] = related.ChildField(
        AnnotationHandlerWriteOutputBoxSnippetsConfig, required=False, default=None
    )

    random_state: Optional[int] = related.ChildField(cls=int, default=None, required=False)


@related.mutable(strict=True)
class AnnotationHandlerConfig(BaseConfigClass):
    """Class for parsing information from yaml in respective hierarchy"""

    class_mapping: Optional[ClassMappingConfig] = related.ChildField(
        cls=ClassMappingConfig, required=False, default=None
    )

    pascal_voc_input_data: List[AnnotationHandlerPASCALVOCInputDataConfig] = related.SequenceField(
        AnnotationHandlerPASCALVOCInputDataConfig, default=[]
    )

    coco_input_data: List[AnnotationHandlerSingleFileInputDataConfig] = related.SequenceField(
        AnnotationHandlerSingleFileInputDataConfig, default=[]
    )

    cvat_input_data: List[AnnotationHandlerSingleFileInputDataConfig] = related.SequenceField(
        AnnotationHandlerSingleFileInputDataConfig, default=[]
    )

    write_output: Optional[AnnotationHandlerWriteOutputConfig] = related.ChildField(
        AnnotationHandlerWriteOutputConfig, required=False, default=None
    )

    reduction_class_mapping: Optional[ReductionMappingConfig] = related.ChildField(
        cls=ReductionMappingConfig, required=False, default=None
    )

    mot_input_data: List[AnnotationHandlerMOTInputDataConfig] = related.SequenceField(
        AnnotationHandlerMOTInputDataConfig, default=[]
    )



# Copyright 2021 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""Module for different utility operations regarding detection models"""
import logging
import os
from typing import Optional, Tuple

from mlcvzoo_base.api.configuration import BaseDetectorConfig

logger = logging.getLogger(__name__)


def parse_model_info(yaml_config_path: Optional[str]) -> Tuple[str, str, str]:
    """
    Parses model configuration to specific information: model_specifier, model_type, model_version

    Args:
        yaml_config_path: String, defines the path where the model configuration remains

    Returns: Tuple of model information: model_specifier, model_type, model_version

    """

    if yaml_config_path is None:
        model_specifier, model_type, model_version = "", "", ""
    elif os.path.isfile(yaml_config_path):
        model_specifier = os.path.basename(yaml_config_path).replace(".yaml", "")

        model_position = model_specifier.find("_")

        model_type = model_specifier[0:model_position]
        model_version = model_specifier[model_position + 1 :]
    else:
        model_specifier, model_type, model_version = "", "", ""

        logger.debug(
            "WARNING: The given yaml_config_path does not exist! "
            "Therefore the model_info variables will be set to empty strings!"
            "yaml_config_path: '%s'" % yaml_config_path
        )

    return model_specifier, model_type, model_version


def set_model_info(yaml_config_path: Optional[str], base_config: BaseDetectorConfig) -> None:
    """
    Sets model information in the base configuration of a detector.

    Args:
        yaml_config_path: String, defines the path where the model configuration remains
        base_config: BaseDetectorConfig, configuration of a detector

    Returns:
        None
    """

    assert isinstance(base_config, BaseDetectorConfig)

    model_specifier, model_type, model_version = parse_model_info(
        yaml_config_path=yaml_config_path
    )

    if base_config.MODEL_SPECIFIER == "":
        base_config.MODEL_SPECIFIER = model_specifier

    if base_config.MODEL_TYPE == "":
        base_config.MODEL_TYPE = model_type

    if base_config.MODEL_VERSION == "":
        base_config.MODEL_VERSION = model_version



# Copyright 2021 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""
Module for parsing information from yaml in python accessible attributes for object
detection classes.
"""
from __future__ import annotations

from dataclasses import dataclass

import related
from config_builder import BaseConfigClass

from mlcvzoo_base.api.configuration import (
    BaseDetectorConfig,
    BaseDetectorInferenceConfig,
)


@dataclass
@related.mutable(strict=True)
class DetectorConfig(BaseConfigClass, BaseDetectorConfig):
    """Class for parsing information about detector model"""

    # NOTE: When the fields are empty, they are filled based on the given config-path filename

    MODEL_TYPE: str = related.StringField(default="")
    MODEL_VERSION: str = related.StringField(default="")
    MODEL_SPECIFIER: str = related.StringField(default="")

    def __call__(self) -> DetectorConfig:
        return DetectorConfig(
            MODEL_VERSION="",
            MODEL_TYPE="",
            MODEL_SPECIFIER="",
        )


@related.mutable(strict=True)
class InferenceConfig(BaseConfigClass, BaseDetectorInferenceConfig):
    """Class for parsing information about object detection inferences"""

    config_path: str = related.StringField()
    checkpoint_path: str = related.StringField()
    score_threshold: float = related.FloatField()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mlcvzoo_base',
 'mlcvzoo_base.api',
 'mlcvzoo_base.api.data',
 'mlcvzoo_base.configuration',
 'mlcvzoo_base.data_preparation',
 'mlcvzoo_base.data_preparation.annotation_builder',
 'mlcvzoo_base.data_preparation.annotation_parser',
 'mlcvzoo_base.data_preparation.annotation_writer',
 'mlcvzoo_base.evaluation',
 'mlcvzoo_base.evaluation.object_detection',
 'mlcvzoo_base.metrics',
 'mlcvzoo_base.metrics.mlflow',
 'mlcvzoo_base.models',
 'mlcvzoo_base.models.read_from_file',
 'mlcvzoo_base.third_party',
 'mlcvzoo_base.third_party.efficientdet_pytorch',
 'mlcvzoo_base.third_party.imutils',
 'mlcvzoo_base.third_party.py_faster_rcnn',
 'mlcvzoo_base.utils']

package_data = \
{'': ['*']}

install_requires = \
['dataclasses-json>=0.5,<0.6',
 'gitpython>=3',
 'imageio>=2.9,<3.0',
 'mlflow>=1.22,<2.0',
 'nptyping>=2.0,<3.0',
 'numpy>=1.19.2,!=1.19.5',
 'opencv-contrib-python>=4.5,<5.0,!=4.5.5.64',
 'opencv-python>=4.5,<5.0,!=4.5.5.64',
 'pillow>=8.2,<10.0',
 'protobuf<=3.20',
 'related-mltoolbox>=1.0,<2.0',
 'setuptools',
 'tensorboardX>=2.5,<3.0',
 'terminaltables>=3.1,<4.0',
 'tqdm>=4.61,<5.0',
 'yaml-config-builder>=6,<8']

setup_kwargs = {
    'name': 'mlcvzoo-base',
    'version': '4.5.0',
    'description': 'MLCVZoo Base Package',
    'long_description': '# MLCVZoo Base\n\nThe MLCVZoo is an SDK for simplifying the usage of various (machine learning driven)\ncomputer vision algorithms. The package **mlcvzoo_base** provides the base modules\nthat are defining the MLCVZoo API. Furthermore, it includes modules that allow to handle\nand process the data structures of the MLCVZoo, as well as providing modules for\nrunning evaluations / calculation of metrics.\n\nFurther information about the MLCVZoo can be found [here](../README.md).\n\n## Install\n`\npip install mlcvzoo-base\n`\n\n## Technology stack\n\n- Python\n',
    'author': 'Maximilian Otten',
    'author_email': 'maximilian.otten@iml.fraunhofer.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://git.openlogisticsfoundation.org/silicon-economy/base/ml-toolbox/mlcvzoo',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# Copyright 2021 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""
Module for configuring the parsing of information from yaml in python
accessible attributes for the ReadFromFileModel class
"""
from typing import Dict, List, Optional

import related
from config_builder import BaseConfigClass

from mlcvzoo_base.api.configuration import Configuration
from mlcvzoo_base.configuration.annotation_handler_config import AnnotationHandlerConfig
from mlcvzoo_base.configuration.class_mapping_config import ClassMappingConfig
from mlcvzoo_base.configuration.detector_configs import DetectorConfig
from mlcvzoo_base.configuration.reduction_mapping_config import ReductionMappingConfig


@related.mutable(strict=True)
class ReadFromFileConfig(BaseConfigClass, Configuration):
    """Class for parsing information from yaml in respective hierarchy"""

    mutual_attribute_map: Dict[str, List[str]] = dict()

    class_mapping: ClassMappingConfig = related.ChildField(ClassMappingConfig)

    annotation_handler_config: AnnotationHandlerConfig = related.ChildField(
        cls=AnnotationHandlerConfig
    )

    use_image_name_hash: bool = related.BooleanField(default=False)

    include_segmentations: bool = related.BooleanField(default=False)

    reduction_class_mapping: Optional[ReductionMappingConfig] = related.ChildField(
        cls=ReductionMappingConfig, required=False, default=None
    )

    base_config: DetectorConfig = related.ChildField(cls=DetectorConfig, default=DetectorConfig())
